;
; Joystick Monitor v1.0
;   by Joel "Quadko" McIntyre - www.joelmcintyre.com
;
; Source and binary released to public domain
;
; Program to test joystick setup and behavior in DOSBox - joystick 1/2, gamepads, etc.
; Uses BIOS calls to retrieve joystick positions and buttons
;   -  In real DOS, BIOS is slowest way to retrieve joystick information
;   -  In DOSBox, BIOS is fastest way because it is directly emulated, just a data copy
;            All other methods have to be emulated and take more processing power
;            Speed measurements acquired via JoyCalib from oldskool.org PC Software section
;            Kudos to Trixter, oldskool.org admin, DOS maven extraordinaire and DOS retro community pillar
;            Forums: www.DosGames.com/forum   www.Vintage-Computer.com/forum/vcforum.php
;
; NB: DOSBox joystick axes are auto-calibrated and  have range 01 - FE with 7F/80 in center
;     To support real dos we'd have to add calibration code at the beginning, and adjust real
;     range to virtual range for min/max X & Y & X2 & Y2 scales. That would be a pain; kudos
;     to real dos joystick programmers! :) We're just letting DOSBox do our calibration for us.
;
; This assembly written for TASM5 to produce an exe file, assembled using commands:
;    TASM  JoyMon
;    TLINK JoyMon
;

;   Setup memory model and defaults for COM file
	org 100h
	.model tiny
	.stack
	
	.code
	.startup
; program start

	; load data segment into ds (tiny model, ds = cs at start, but that's not what we need)
	mov ax, @data
	mov ds, ax
	
	; clear screen by resetting video mode
	mov ah, 0h
	mov al, 03h
	int 10h ; clear screen
	
	; output newlines to move cursor down 20 lines (bypassing top of screen  joystick/game area)
	lea dx, NewLine20
	mov ah, 9
	int 21h ; print string

	; output the Title/help message "Joystick Monitor - Press any key to exit"
	lea dx, Message
	mov ah, 9
	int 21h ; print string

	; Hide the cursor so it doesn't flash all over the screen
	mov cx, 2607h 
	mov ah, 1
	int 10h ; set/hide cursor
	
    ; game loop, loops until keypress	
looptop:
	
    ; get joystick position values from DOS
	mov ah, 84h
	mov dx, 1
	int 15h ; read gameport analog positions
	
	; Save joystick X & Y to stack (twice)
	push dx; Y2
	push cx; X2
	push bx ; Y
	push ax ; X
	push dx; Y2 again
	push cx; X2 again
	push bx ; Y again
	push ax ; X again
	
	;
	; Calculate new X & Y  1 & 2 Positions ====================================================
	;
	
	; restore joystick X from stack
	pop bx
	shr bx, 4 ; X / 16
	shl bx, 2 ; bx * 4

	inc bx         ; increment x by 4 bytes (1 position) to show as axis
	inc bx
	inc bx         
	inc bx
	mov NXPos, bx
	
	; restore joystick Y from stack
	pop bx
	shr bx, 4 ; Y / 16
	
	mov al, 0A0h ; ax = bl * 160 (A0h)
	mul bl
	
	add ax, 0A0h  ; move to second line of text
	mov NYPos, ax

	; restore joystick X2 from stack
	pop bx
	shr bx, 4 ; X2 / 16
	shl bx, 2 ; bx * 4

	inc bx         ; increment x2 by 4 bytes (1 position) to show as axis
	inc bx
	inc bx         
	inc bx
	add bx, 80 ; second joystick is in column on right of screen
	mov NXPos2, bx
	
	; restore joystick Y2 from stack
	pop bx
	shr bx, 4 ; Y / 16
	
	mov al, 0A0h ; ax = bl * 160 (A0h)
	mul bl
	
	add ax, 0A0h  ; move to second line of text
	add ax, 80 ; second joystick is in column on right of screen
	mov NYPos2, ax
	
	;
	; Blank then Draw Joystick Position on screen, X & Y ====================================================
	;
	
	; Note: Don't blank positions if value didn't change
		
	mov cx, 0B800h ; first line of text into es
	mov es, cx
	mov ax, 00720h      ; space character ' ' bgcolor 0, color 7 (normal white on black)
	
	; First, blank out old positions
	;X
	mov cx, NXPos
	mov bx, PXPos
	cmp cx, bx
	je SkipBlankX
	mov di, bx
	
	mov es:[di], ax
	inc di
	inc di
	mov es:[di], ax
SkipBlankX:

	; Y
	mov cx, NYPos
	mov bx, PYPos
	cmp cx, bx
	je SkipBlankY
	mov di, bx
	
	mov es:[di], ax
	inc di
	inc di
	mov es:[di], ax
SkipBlankY:

	; X2
	mov cx, NXPos2
	mov bx, PXPos2
	cmp cx, bx
	je SkipBlankX2
	mov di, bx
	
	mov es:[di], ax
	inc di
	inc di
	mov es:[di], ax
SkipBlankX2:
	
	; Y2
	mov cx, NYPos2
	mov bx, PYPos2
	cmp cx, bx
	je SkipBlankY2
	mov di, bx
	
	mov es:[di], ax
	inc di
	inc di
	mov es:[di], ax
SkipBlankY2:
	
	; Find center location and blank crosshairs - is just X + Y 
	mov ax, PXPos
	mov bx, PYPos
	add ax, bx
	
	mov bx, NXPos
	mov cx, NYPos
	add bx, cx
	
	cmp ax, bx
	je SkipBlankXY
	
	mov di, ax 
	
	mov ax, 0420h ; ' ' char (20h) and red on black
	mov es:[di], ax  
	inc di
	inc di
	mov es:[di], ax
SkipBlankXY:
	
	; Find center location and blank crosshairs - is just X2 + Y2 - but have to -80
	mov ax, PXPos2
	mov bx, PYPos2
	sub bx, 80 ; X & Y are both offset, so remove one offset
	add ax, bx
	
	mov bx, NXPos2
	mov cx, NYPos2
	sub cx, 80 ; X & Y are both offset, so remove one offset
	add bx, cx
	
	cmp ax, bx
	je SkipBlankXY2
	
	mov di, ax 
	
	mov ax, 0420h ; ' ' char (20h) and red on black
	mov es:[di], ax  
	inc di
	inc di
	mov es:[di], ax
SkipBlankXY2:
	
	;
	; == Draw Joystick Locations ==================================================
	;
	
	; No harm to draw on top of old position if it's the same, so no 'Skip' checks as in blanking code

	mov ax, 00FDBh ; solid block character, white on black	
	
	; X
	mov di, NXPos

	mov es:[di], ax  
	inc di
	inc di
	mov es:[di], ax
	
	; Y from stack
	mov di, NYPos

	mov es:[di], ax  
	inc di
	inc di
	mov es:[di], ax
	
	mov ax, 00EDBh ; solid block character, yellow on black	
	
	; X2
	mov di, NXPos2

	mov es:[di], ax  
	inc di
	inc di
	mov es:[di], ax
	
	; Y2
	mov di, NYPos2

	mov es:[di], ax  
	inc di
	inc di
	mov es:[di], ax
	
	
	; Find center location and draw crosshairs - is just X + Y
	mov ax, NXPos
	mov bx, NYPos
	add ax, bx
	
	mov di, ax 
	
	mov ah, 04h ; red on black
	mov al, '>' ; half of crosshair ><

	mov es:[di], ax  
	inc di
	inc di
	mov al, '<' ; half of crosshair ><
	mov es:[di], ax
	
	; Find center location and draw crosshairs - is just X + Y
	mov ax, NXPos2
	mov bx, NYPos2
	sub bx, 80 ; X & Y are both offset for column, remove one offset
	add ax, bx
	
	mov di, ax 
	
	mov ah, 02h ; green on black
	mov al, '>' ; half of crosshair ><

	mov es:[di], ax  
	inc di
	inc di
	mov al, '<' ; half of crosshair ><
	mov es:[di], ax
	
	;
	; == print the joystick x/y position w/ space between ==================================================
	;
	
	; Print "Joystick 1: [X] [Y]", up to 4 hex characters for values, popping X & Y
	lea dx, Joystick
	mov ah, 9
	int 21h ; print string
	
	mov ah, 2
	mov dl, '1'
	int 21h ; print single character
	
	lea dx, JoyCol
	mov ah, 9
	int 21h ; print string
	
	pop ax
	call PrintHex16
	
	mov dl, ' '
	mov ah, 2
	int 21h ; print single character
	
	pop ax
	call PrintHex16	
	
	; output a few spaces between Joystick 1 & 2
	lea dx, JoySpace
	mov ah, 9
	int 21h ; print string
	
	; Print "Joystick 2: [X2] [Y2]", up to 4 hex characters for values, popping X2 & Y2
	; output the JoyStart "Joystick " message
	
	lea dx, Joystick
	mov ah,9
	int 21h
	
	mov ah, 2
	mov dl, '2'
	int 21h
	
	lea dx, JoyCol
	mov ah,9
	int 21h
	
	pop ax
	call PrintHex16	
	
	mov dl, ' '
	mov ah,2
	int 21h
	
	pop ax
	call PrintHex16	
	
	; print the "Buttons: " string
	
	lea dx, JoyButtons
	mov ah, 9
	int 21h ; print string
	
	;
	; Read and output Joystick Buttons ==========================================================
	;
	
	; get joystick button values from DOS
	mov al, 0
	mov ah, 84h
	mov dx, 0
	int 15h
	
	;
	; Output Button String
	;
	
	; Print 0/1 for each of 4 buttons by creating string in Btn data
	mov bx, ax
	shr bx, 4h
	lea di, ButtonDB
	
	; button 1
	mov al, '1'
	mov cx, bx
	and cx, 1
	
	jz NextB1
	mov al, 249 ; blank it out
NextB1:
    mov [di], al
	
	shr bx, 1 ; check next bit
	inc di    ; move to next position
	
	; button 2
	mov al, '2'
	mov cx, bx
	and cx, 1
	
	jz NextB2
	mov al, 249 ; blank it out
NextB2:
    mov [di], al
	
	shr bx, 1 ; check next bit
	inc di    ; move to next position
	
	; button 3
	mov al, '3'
	mov cx, bx
	and cx, 1
	
	jz NextB3
	mov al, 249 ; blank it out
NextB3:
    mov [di], al
	
	shr bx, 1 ; check next bit
	inc di    ; move to next position
	
	; button 4
	mov al, '4'
	mov cx, bx
	and cx, 1
	
	jz NextB4
	mov al, 249 ; blank it out
NextB4:
    mov [di], al
	
	inc di    ; move to next position
	
	; end string
	mov cl, '$' ; dos string terminator
	mov [di], cl
	
	; print button string we've just built in ButtonDB variable
	lea dx, ButtonDB
	mov ah, 9
	int 21h ; print string
		
	; write some blank spaces to overwrite any earlier characters
	; (variable length string depending on joystick values), then use the
	; Carriage Return Trick (output 13h) to return cursor to beginning of line
	; without including a linefeed (10h) to move cursor to next line - lets us 
	; easily write a status line without having to move the cursor around the screen
	; Cheating, but the best kind of simple clever cheating. :)
	lea dx, JoyEnd
	mov ah, 9
	int 21h ; print string
	
	; copy joystick current 'NEW' position to 'PREVIOUS' position now we are done with loop
	mov ax, NXPos
	mov PXPos, ax
	mov ax, NYPos
	mov PYPos, ax
	mov ax, NXPos2
	mov PXPos2, ax
	mov ax, NYPos2
	mov PYPos2, ax
	
	; was a key pressed? if not, loop top, otherwise exit program
	mov ah, 0Bh
	int 21h ; Get keypress/buffer state
	
	cmp al, 0 ; if al == 0, no keypress
	jnz skiploop
	jmp looptop ; return to top of loop and do it again!
skiploop:
	
	; read the key to clear keyboard buffer, otherwise dos prompt will display character, that's messy and unkind
	mov ah, 08h
	int 21h	; read key
	
	; clear screen by resetting video mode (hopefully to default mode)
	mov ah, 0h
	mov al, 03h
	int 10h ; set video mode

	; exit program	
	mov ah, 4Ch
	mov al, 0
	int 21h ; dos exit program
	
;================================================================================================
;== FUNCTIONS
;================================================================================================

;-- PrintHex16 ---------------------------------------------------------------------	
; DESC: print out 4 hex digits for WORD value in AX, don't print leading zeros
; PARAMETERS:
;    AX: Value to print
;
; RETURNS:
;    None
;
PrintHex16:

	lea di, PrintHex16DB ; use variable PrintHex16DB as our buffer
	
	; work backwards, start at 5th character of buffer, 
	; insert dos string terminator character $
	add di, 4
	mov byte ptr [di], '$'
	dec di 
	
	; first byte character, either add 0 or A (A-10) to nibble value
	mov bx, ax ; Always print first character, even if zero
	and bl, 0fh
	cmp bl, 9
	jg hex1
	add bl, '0'
	jmp done1
hex1:
    add bl, 37h ; 'A' (41) - 10 = 31
done1:
	mov [di], bl ; write character to buffer, move to previous position
	dec di
	
	; second byte character, either add 0 or A (A-10) to nibble value
	shr ax, 04h
	jz done ; if remaining value is zero, don't print more characters
	mov bx, ax
	and bl, 0fh
	cmp bl, 9
	jg hex2
	add bl, '0'
	jmp done2
hex2:
    add bl, 37h ; 'A' (41) - 10 = 31
done2:
	mov [di], bl ; write character to buffer, move to previous position
	dec di
	
	; third byte character, either add 0 or A (A-10) to nibble value
	shr ax, 04h
	jz done ; if remaining value is zero, don't print more characters
	mov bx, ax
	and bl, 0fh
	cmp bl, 9
	jg hex3
	add bl, '0'
	jmp done3
hex3:
    add bl, 37h ; 'A' (41) - 10 = 31
done3:
	mov [di], bl ; write character to buffer, move to previous position
	dec di
	
	; forth byte character, either add 0 or A (A-10) to nibble value
	shr ax, 04h
	jz done ; if remaining value is zero, don't print more characters
	mov bx, ax
	and bl, 0fh
	cmp bl, 9
	jg hex4
	add bl, '0'
	jmp done4
hex4:
    add bl, 37h ; 'A' (41) - 10 = 31
done4:
	mov [di], bl ; write character to buffer, move to previous position
	dec di

done:
    inc di ; move to next position (last character written)
	
	; output the numeric value message
	mov dx, di
	mov ah, 9
	int 21h ; print string
	
	ret
; END PROCEDURE PrintHex16

;================================================================================================
;== DATA VARIABLES
;================================================================================================

; DATA segment 
	.data
Message	DB 'Joystick Monitor, press any key to exit',13,10,'$' ; title string

; substrings for joystick message parts
Joystick	DB 'Joystick ','$'
JoyCol	DB ': ','$'
JoySpace	DB '  ','$'
JoyButtons	DB ' buttons: ','$'
JoyEnd	DB '                       ',13,'$'

; newline characters - easily move cursor down 20 lines with newline
NewLine20 DB 10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,'$'

; working area for printing button and PrintHex16 function
ButtonDB DB '-------','$'
PrintHex16DB DB '---------','$'

; previous values
PXPos DW 0
PYPos DW 0
PXPos2 DW 0
PYPos2 DW 0

; current/new values
NXPos DW 0
NYPos DW 0
NXPos2 DW 0
NYPos2 DW 0

; EOF marker for TASM
END

